# Copyright (c) 2015-2016 by Farsight Security, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import socket
import threading

import axa
import nmsg

_read_timeout = 0.1

class Relay:
    """
    Relays messages from a Client to a nmsg input suitable for use
    with a nmsg.io.
    """
    def __init__(self, client):
        """
        Args:
            client (axa.Client): connected Client object
        """
        self._client = client

        _sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        _sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        _sock.bind(('127.0.0.1', 0))

        self.input = nmsg.input.open_sock(*_sock.getsockname())
        self._output = nmsg.output.open_sock(*_sock.getsockname())

        self._t = threading.Thread(target=self._run)
        self._t.setDaemon(True)
        self._stop_cond = threading.Condition()

    def isAlive(self):
        """
        Returns whether or not the enclosed relay thread is alive.
        """
        return self._t.isAlive()

    def start(self):
        """
        Starts the enclosed relay thread.
        """
        self._t.start()

    def join(self, timeout=None):
        """
        Stops and joins the Relay thread.
        """
        self.stop()
        self._t.join(timeout=timeout)

    def stop(self):
        """
        Signals the Relay thread to stop.
        """
        self._stop_cond.set()

    def _run(self):
        while not self._stop_cond.isSet():
            try:
                msg = self._client.recv(delay=_read_timeout)
            except axa.Timeout:
                continue

            if isinstance(msg, axa.MsgWhitNmsg):
                self._output.write(msg.nmsg_obj())
