# Copyright (c) 2015-2016 by Farsight Security, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""An SRA or RAD client that watches SIE channels using the pyaxa wrappers
for libaxa to get AXA hits."""

import axa

import time


class Server(object):
    """Connect to an SRA or RAD server analogous to sratunnel"""
    def __init__(self, server, parms, rlimit=None,
                 persist=True, debug=0, trace=0, rad=False):
        """
        Args:
            server (str): 'unix:/path/to/socket', 'tcp:hostname,port',
                or 'ssh:[user@]host'
            parms (list): MsgWatch and MsgChannel or MsgAnom parameters
            persist (bool): True restores the connection after connection or
                other recoverable failures, but the default is False.
            debug (int): debugging level between 0 and DEBUG_MAX; default=0
            trace (int): tracing level for the SRA or RAD server; default=0

        Raises:
            FatalError: invalid parameters
        """

        self.client = axa.Client(rad, debug)
        self.debug = debug if int(debug) else 0
        if trace:
            self.trace = axa.MsgTrace(rad, None, int(trace))
        else:
            self.trace = None
        self.server = server
        self.parms = parms
        if rlimit is None:
            self.rlimit = None
        else:
            self.rlimit = axa.MsgRlimit(rad, None, rlimit)
        self.persist = persist

    def __repr__(self):
        result = ("%s%s client of %s\n    %s\n" %
                  ('persistent ' if self.persist else '',
                   'RAD' if self.client.rad else 'SRA',
                   self.server,
                   [str(p) for p in self.parms]))
        prefix = '    '
        if self.rlimit:
            result += '%s%s' % (prefix, str(self.rlimit))
            prefix = '  '
        if self.debug:
            result += "%sdebug=%d" % (prefix, self.debug)
            prefix = '  '
        if self.trace:
            result += "%strace=%d" % (prefix, self.trace.level)
        return result

    def open(self):
        """(Re)connect to the server, set watches or anomaly modules,
        and turn on channels.

        Raises:
            RetryError: non-fatal sending problems that should be recovered
                by re-connecting to the server
            FatalError: unrecoverable problem
        """

        while True:
            # Start by closing the connection to the server and waiting until
            #   it is time to try again.
            ms = self.client.again()
            if ms > 0:
                ms = ms/1000.0
                if (self.debug and (ms >= 0.1
                                    or self.debug >= axa.DEBUG_TRACE)):
                    axa.trace('try connection to %s in %.1f seconds'
                              % (self.server, ms))
                time.sleep(ms)

            try:
            # Try to open the connection.
                self.client.open(self.server)

                # Turn on tracing immediately
                if self.trace:
                    self.client.cmd(self.trace)

                # Stop data until everything is configured.
                self.client.cmd(axa.MsgCmd(self.client.rad, None,
                                           axa.P_OP_PAUSE))

                # Send the watch and channel parameters to the SRA server
                for p in self.parms:
                    self.client.cmd(p, exact=True)

                # Send the rate limit if we have one.
                if self.rlimit:
                    self.client.cmd(self.rlimit)

                # Let the data stream.
                self.client.cmd(axa.MsgCmd(self.client.rad, None,
                                           axa.P_OP_GO))

                # reset the back-off timer for self.client.again() after a
                #   successful set of commands
                self.client.backoff_reset()

                # Quit with the connection open.
                return

            except axa.RetryError as except_msg:
                if not self.persist:
                    raise
                if self.debug != 0:
                    print except_msg


    def recv(self, delay=None):
        """Read messages from the SRA server."""
        while True:
            try:
                return self.client.recv()
            except axa.RetryError as except_msg:
                if not self.persist:
                    raise
                if self.debug != 0:
                    print except_msg
                self.open()

    def mgmt(self):
        self.client.cmd(axa.MsgCmd(self.client.rad, None,
                                           axa.P_OP_MGMT_GET))
